particle_types = require ("particle-rules")

screen = {
		w = love.graphics.getWidth(),
		h = love.graphics.getHeight()
	}

G = 10^5
amount = 200
speed_max = 80
min_dist = 10

function get_color (particle_type)
	local r = 0.5+(particle_type[2]+1)/2
	local g = 0.5+(particle_type[3]+1)/2
	local b = 0.5+(particle_type[4]+1)/2
	local a = 1
	return {r,g,b,a}
end

function particle_types_interaction (p1, p2)
	local interaction = 0
	if p2.particle_type[1] == "a" then
		interaction = p2.particle_type[2]
	elseif p2.particle_type[1] == "b" then
		interaction = p2.particle_type[3]
	elseif p2.particle_type[1] == "c" then
		interaction = p2.particle_type[4]
	end
	return interaction
end

function love.load()
	love.math.setRandomSeed(love.timer.getTime())

	particles = {}
	interactions = {}
	for id_particle = 1, amount do
		local particle_type = particle_types[math.random (#particle_types)]
		local color = get_color (particle_type)
		local size = 2
		local x = math.random(size+10, screen.w-size)
		local y = math.random(size+10, screen.h-size)
		particles[id_particle] = {x=x, y=y, size = size, speed={x=0,y=0}, 
			force={x=0,y=0}, last_force={x=0,y=0}, 
			color = color, particle_type = particle_type}
	end
	print ('particles: ' .. #particles)
	
	for id_p1 = 1, (amount-1) do
		for id_p2 = (id_p1+1), amount do
			local p1 = particles[id_p1]
			local p2 = particles[id_p2]
			if p1 and p2 then
				local int1 = particle_types_interaction (p1, p2)
				local int2 = particle_types_interaction (p2, p1)
				interactions[#interactions+1] = {id_p1, id_p2, int1, int2}
			else
				print ('no interaction: ' .. id_p1..'-'..id_p2)
			end
		end
	end
	print ('interactions: ' .. #interactions)
end

function update_force (interaction, dt)
	local p1 = particles[interaction[1]]
	local p2 = particles[interaction[2]]
	local int1 = interaction[3]
	local int2 = interaction[4]
	local dx = p1.x-p2.x
	local dy = p1.y-p2.y
	local sd = (dx^2 + dy^2) -- distance is already square
	if sd > (min_dist^2) then
		local alpha = math.atan2(dy, dx)
		local m1 = p1.size
		local m2 = p2.size
	--	local force = G*m1*m2/sd
	--	print (dx)
		local f_x = G*m1*m2/(sd) * math.cos(alpha)
		local f_y = G*m1*m2/(sd) * math.sin(alpha)
		
		p1.force.x = p1.force.x - f_x * int1
		p1.force.y = p1.force.y - f_y * int1
		
		p2.force.x = p2.force.x + f_x * int2
		p2.force.y = p2.force.y + f_y * int2
	end
end

function update_speed (particle, dt)
	local vx = particle.speed.x + dt*particle.force.x/(particle.size)^2
	local vy = particle.speed.y + dt*particle.force.y/(particle.size)^2
	local speed = math.sqrt(vx^2 + vy^2)
	if speed > speed_max then
		vx = vx * (speed_max/speed)
		vy = vy * (speed_max/speed)
	end
	particle.speed.x = vx
	particle.speed.y = vy
	particle.last_force = {x=particle.force.x, y=particle.force.y}
	particle.force = {x=0,y=0}
--	print (particle.speed.x)
end

function update_position (particle, dt)

	if particle.x < (particle.size) then 
		particle.speed.x = math.abs(particle.speed.x )/2
	elseif particle.x > (screen.w-particle.size) then
		particle.speed.x = -math.abs(particle.speed.x )/2
	end
	if particle.y < (particle.size) then 
		particle.speed.y = math.abs(particle.speed.y)/2
	elseif particle.y > (screen.h-particle.size) then
		particle.speed.y = -math.abs(particle.speed.y)/2
	end
	particle.x = particle.x + dt*particle.speed.x
	particle.y = particle.y + dt*particle.speed.y
end

function love.update(dt)
	for id_interaction, interaction in pairs (interactions) do
		update_force (interaction, dt)
	end
	
	for id_particle, particle in pairs (particles) do
		update_speed (particle, dt)
		update_position (particle, dt)
	end
	
end


function love.draw()
	for id_particle, particle in pairs (particles) do
		love.graphics.setColor(particle.color)
		love.graphics.circle("fill", particle.x, particle.y, particle.size)

		love.graphics.line(particle.x, particle.y, particle.x-particle.speed.x/10, particle.y-particle.speed.y/10)
		
		-- forces, just disabled
--		love.graphics.setColor(1,0,0,1)
--		love.graphics.line(particle.x, particle.y, particle.x+particle.last_force.x*10^(-2), particle.y+particle.last_force.y*10^(-2))

		love.graphics.setColor(1,1,1,1)
		love.graphics.points(particle.x, particle.y)
	end
	love.graphics.setColor(1,1,1,1)
	love.graphics.print("Current FPS: "..tostring(love.timer.getFPS( )), 10, 10)
--	print(particles[1].x)
end